@@ -16,6 +16,7 @@
 #include "Kernel.h"
 #include "KernelInvocation.h"
 #include "Memory.h"
+#include "Program.h"
 #include "WorkGroup.h"
 #include "WorkItem.h"
 
@@ -24,6 +25,7 @@ using namespace std;
 
 struct
 {
+  int id;
   WorkGroup *workGroup;
   WorkItem  *workItem;
 } static THREAD_LOCAL workerState;
@@ -45,23 +47,16 @@ KernelInvocation::KernelInvocation(const Context *context, const Kernel *kernel,
   m_numGroups.x = m_globalSize.x/m_localSize.x;
   m_numGroups.y = m_globalSize.y/m_localSize.y;
   m_numGroups.z = m_globalSize.z/m_localSize.z;
+  if (!m_kernel->getProgram()->requiresUniformWorkGroups())
+  {
+    m_numGroups.x += m_globalSize.x % m_localSize.x ? 1 : 0;
+    m_numGroups.y += m_globalSize.y % m_localSize.y ? 1 : 0;
+    m_numGroups.z += m_globalSize.z % m_localSize.z ? 1 : 0;
+  }
 
   // Check for user overriding number of threads
-  m_numWorkers = 0;
-  const char *numThreads = getenv("OCLGRIND_NUM_THREADS");
-  if (numThreads)
-  {
-    char *next;
-    m_numWorkers = strtoul(numThreads, &next, 10);
-    if (strlen(next))
-    {
-      cerr << "Oclgrind: Invalid value for OCLGRIND_NUM_THREADS" << endl;
-    }
-  }
-  else
-  {
-    m_numWorkers = thread::hardware_concurrency();
-  }
+  m_numWorkers = getEnvInt("OCLGRIND_NUM_THREADS",
+                           thread::hardware_concurrency(), false);
   if (!m_numWorkers || !m_context->isThreadSafe())
     m_numWorkers = 1;
 
@@ -151,23 +146,6 @@ void KernelInvocation::run(const Context *context, Kernel *kernel,
                            Size3 globalSize,
                            Size3 localSize)
 {
-  try
-  {
-    // Allocate and initialise constant memory
-    kernel->allocateConstants(context->getGlobalMemory());
-  }
-  catch (FatalError& err)
-  {
-    ostringstream info;
-    info << "OCLGRIND FATAL ERROR "
-         << "(" << err.getFile() << ":" << err.getLine() << ")"
-         << endl << err.what()
-         << endl << "When allocating kernel constants for '"
-         << kernel->getName() << "'";
-    context->logError(info.str().c_str());
-    return;
-  }
-
   // Create kernel invocation
   KernelInvocation *ki = new KernelInvocation(context, kernel, workDim,
                                               globalOffset,
@@ -180,9 +158,6 @@ void KernelInvocation::run(const Context *context, Kernel *kernel,
   context->notifyKernelEnd(ki);
 
   delete ki;
-
-  // Deallocate constant memory
-  kernel->deallocateConstants(context->getGlobalMemory());
 }
 
 void KernelInvocation::run()
@@ -194,7 +169,7 @@ void KernelInvocation::run()
   vector<thread> threads;
   for (unsigned i = 0; i < m_numWorkers; i++)
   {
-    threads.push_back(thread(&KernelInvocation::runWorker, this));
+    threads.push_back(thread(&KernelInvocation::runWorker, this, i));
   }
 
   // Wait for workers to complete
@@ -204,10 +179,16 @@ void KernelInvocation::run()
   }
 }
 
-void KernelInvocation::runWorker()
+int KernelInvocation::getWorkerID() const
+{
+  return workerState.id;
+}
+
+void KernelInvocation::runWorker(int id)
 {
   workerState.workGroup = NULL;
   workerState.workItem = NULL;
+  workerState.id = id;
   try
   {
     while (true)
@@ -227,7 +208,17 @@ void KernelInvocation::runWorker()
           // No more work to do
           break;
 
-        workerState.workGroup = new WorkGroup(this, m_workGroups[index]);
+        Size3 wgid   = m_workGroups[index];
+        Size3 wgsize = m_localSize;
+
+        // Handle remainder work-groups
+        for (unsigned i = 0; i < 3; i++)
+        {
+          if (wgsize[i]*(wgid[i]+1) > m_globalSize[i])
+            wgsize[i] = m_globalSize[i] % wgsize[i];
+        }
+
+        workerState.workGroup = new WorkGroup(this, wgid, wgsize);
         m_context->notifyWorkGroupBegin(workerState.workGroup);
       }
 
