@@ -11,6 +11,7 @@
 namespace llvm
 {
   class Function;
+  class LLVMContext;
   class Module;
   class StoreInst;
 }
@@ -50,11 +51,15 @@ namespace oclgrind
     const InterpreterCache* getInterpreterCache(
       const llvm::Function *kernel) const;
     std::list<std::string> getKernelNames() const;
+    llvm::LLVMContext& getLLVMContext() const;
     unsigned int getNumKernels() const;
     const std::string& getSource() const;
     const char* getSourceLine(size_t lineNumber) const;
     size_t getNumSourceLines() const;
+    const TypedValue& getProgramScopeVar(const llvm::Value *var) const;
+    size_t getTotalProgramScopeVarSize() const;
     unsigned long getUID() const;
+    bool requiresUniformWorkGroups() const;
 
   private:
     Program(const Context *context, llvm::Module *module);
@@ -67,9 +72,16 @@ namespace oclgrind
     const Context *m_context;
     std::vector<std::string> m_sourceLines;
 
+    bool m_requiresUniformWorkGroups;
+
+    TypedValueMap m_programScopeVars;
+    size_t m_totalProgramScopeVarSize;
+
     unsigned long m_uid;
     unsigned long generateUID() const;
 
+    void allocateProgramScopeVars();
+    void deallocateProgramScopeVars();
     void pruneDeadCode(llvm::Instruction*);
     void removeLValueLoads();
     void scalarizeAggregateStore(llvm::StoreInst *store);
