@@ -57,6 +57,15 @@ static bool parseArguments(int argc, char *argv[])
       }
       setEnvironment("OCLGRIND_BUILD_OPTIONS", argv[i]);
     }
+    else if (!strcmp(argv[i], "--constant-mem-size"))
+    {
+      if (++i >= argc)
+      {
+        cerr << "Missing argument to --constant-mem-size" << endl;
+        return false;
+      }
+      setEnvironment("OCLGRIND_CONSTANT_MEM_SIZE", argv[i]);
+    }
     else if (!strcmp(argv[i], "--data-races"))
     {
       setEnvironment("OCLGRIND_DATA_RACES", "1");
@@ -73,6 +82,15 @@ static bool parseArguments(int argc, char *argv[])
     {
       outputGlobalMemory = true;
     }
+    else if (!strcmp(argv[i], "--global-mem-size"))
+    {
+      if (++i >= argc)
+      {
+        cerr << "Missing argument to --global-mem-size" << endl;
+        return false;
+      }
+      setEnvironment("OCLGRIND_GLOBAL_MEM_SIZE", argv[i]);
+    }
     else if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help"))
     {
       printUsage();
@@ -86,6 +104,15 @@ static bool parseArguments(int argc, char *argv[])
     {
       setEnvironment("OCLGRIND_INTERACTIVE", "1");
     }
+    else if (!strcmp(argv[i], "--local-mem-size"))
+    {
+      if (++i >= argc)
+      {
+        cerr << "Missing argument to --local-mem-size" << endl;
+        return false;
+      }
+      setEnvironment("OCLGRIND_LOCAL_MEM_SIZE", argv[i]);
+    }
     else if (!strcmp(argv[i], "--log"))
     {
       if (++i >= argc)
@@ -104,6 +131,15 @@ static bool parseArguments(int argc, char *argv[])
       }
       setEnvironment("OCLGRIND_MAX_ERRORS", argv[i]);
     }
+    else if (!strcmp(argv[i], "--max-wgsize"))
+    {
+      if (++i >= argc)
+      {
+        cerr << "Missing argument to --max-wgsize" << endl;
+        return false;
+      }
+      setEnvironment("OCLGRIND_MAX_WGSIZE", argv[i]);
+    }
     else if (!strcmp(argv[i], "--num-threads"))
     {
       if (++i >= argc)
@@ -148,7 +184,7 @@ static bool parseArguments(int argc, char *argv[])
       cout << endl;
       cout << "Oclgrind " PACKAGE_VERSION << endl;
       cout << endl;
-      cout << "Copyright (c) 2013-2016" << endl;
+      cout << "Copyright (c) 2013-2018" << endl;
       cout << "James Price and Simon McIntosh-Smith, University of Bristol"
            << endl;
       cout << "https://github.com/jrprice/Oclgrind" << endl;
@@ -190,40 +226,48 @@ static void printUsage()
     << "       oclgrind-kernel [--help | --version]" << endl
     << endl
     << "Options:" << endl
-    << "     --build-options  OPTIONS  "
-             "Additional options to pass to the OpenCL compiler" << endl
-    << "     --data-races              "
-             "Enable data-race detection" << endl
-    << "     --disable-pch             "
-             "Don't use precompiled headers" << endl
-    << "     --dump-spir               "
-             "Dump SPIR to /tmp/oclgrind_*.{ll,bc}" << endl
-    << "  -g --global-mem              "
-             "Output global memory at exit" << endl
-    << "  -h --help                    "
-             "Display usage information" << endl
-    << "     --inst-counts             "
-             "Output histograms of instructions executed" << endl
-    << "  -i --interactive             "
-             "Enable interactive mode" << endl
-    << "     --log            LOGFILE  "
-             "Redirect log/error messages to a file" << endl
-    << "     --max-errors     NUM      "
-             "Limit the number of error/warning messages" << endl
-    << "     --num-threads    NUM      "
-             "Set the number of worker threads to use" << endl
-    << "     --pch-dir        DIR      "
-             "Override directory containing precompiled headers" << endl
-    << "     --plugins        PLUGINS  "
-             "Load colon separated list of plugin libraries" << endl
-    << "  -q --quick                   "
-             "Only run first and last work-group" << endl
-    << "     --uniform-writes          "
-             "Don't suppress uniform write-write data-races" << endl
-    << "     --uninitialized           "
-             "Report usage of uninitialized values" << endl
-    << "  -v --version                 "
-             "Display version information" << endl
+    << "  --build-options     OPTIONS  "
+          "Additional options to pass to the OpenCL compiler" << endl
+    << "  --constant-mem-size BYTES    "
+          "Change the constant memory size of the device" << endl
+    << "  --data-races                 "
+          "Enable data-race detection" << endl
+    << "  --disable-pch                "
+          "Don't use precompiled headers" << endl
+    << "  --dump-spir                  "
+          "Dump SPIR to /tmp/oclgrind_*.{ll,bc}" << endl
+    << "  --global-mem [-g]            "
+          "Output global memory at exit" << endl
+    << "  --global-mem-size   BYTES    "
+          "Change the global memory size of the device" << endl
+    << "  --help [-h]                  "
+          "Display usage information" << endl
+    << "  --inst-counts                "
+          "Output histograms of instructions executed" << endl
+    << "  --interactive [-i]           "
+          "Enable interactive mode" << endl
+    << "  --local-mem-size    BYTES    "
+          "Change the local memory size of the device" << endl
+    << "  --log               LOGFILE  "
+          "Redirect log/error messages to a file" << endl
+    << "  --max-errors        NUM      "
+          "Limit the number of error/warning messages" << endl
+    << "  --max-wgsize        WGSIZE   "
+          "Change the maximum work-group size of the device" << endl
+    << "  --num-threads       NUM      "
+          "Set the number of worker threads to use" << endl
+    << "  --pch-dir           DIR      "
+          "Override directory containing precompiled headers" << endl
+    << "  --plugins           PLUGINS  "
+          "Load colon separated list of plugin libraries" << endl
+    << "  --quick [-q]                 "
+          "Only run first and last work-group" << endl
+    << "  --uniform-writes             "
+          "Don't suppress uniform write-write data-races" << endl
+    << "  --uninitialized              "
+          "Report usage of uninitialized values" << endl
+    << "  --version [-q]               "
+          "Display version information" << endl
     << endl
     << "For more information, please visit the Oclgrind wiki page:" << endl
     << "-> https://github.com/jrprice/Oclgrind/wiki" << endl
