@@ -8,6 +8,8 @@
 
 #include "core/Plugin.h"
 
+#include <mutex>
+
 namespace llvm
 {
   class Function;
@@ -25,14 +27,24 @@ namespace oclgrind
                                      const TypedValue& result) override;
     virtual void kernelBegin(const KernelInvocation *kernelInvocation) override;
     virtual void kernelEnd(const KernelInvocation *kernelInvocation) override;
-
-    virtual bool isThreadSafe() const override;
+    virtual void workGroupBegin(const WorkGroup *workGroup) override;
+    virtual void workGroupComplete(const WorkGroup *workGroup) override;
 
   private:
     std::vector<size_t> m_instructionCounts;
     std::vector<size_t> m_memopBytes;
     std::vector<const llvm::Function*> m_functions;
 
+    struct WorkerState
+    {
+      std::vector<size_t> *instCounts;
+      std::vector<size_t> *memopBytes;
+      std::vector<const llvm::Function*> *functions;
+    };
+    static THREAD_LOCAL WorkerState m_state;
+
+    std::mutex m_mtx;
+
     std::string getOpcodeName(unsigned opcode) const;
   };
 }
