@@ -14,6 +14,11 @@
 
 #if !defined(_WIN32) || defined(__MINGW32__)
 #include <signal.h>
+#include <unistd.h>
+#else
+#include <io.h>
+#define isatty _isatty
+#define STDIN_FILENO _fileno(stdin)
 #endif
 
 #if HAVE_READLINE
@@ -109,32 +114,40 @@ void InteractiveDebugger::instructionExecuted(
   m_continue     = false;
   m_next         = false;
 
+  bool interactive = isatty(STDIN_FILENO);
   while (true)
   {
     // Prompt for command
     bool eof = false;
     string cmd;
   #if HAVE_READLINE
-    char *line = readline("(oclgrind) ");
-    if (line)
+    if (interactive)
     {
-      cmd = line;
-      free(line);
+      char *line = readline("(oclgrind) ");
+      if (line)
+      {
+        cmd = line;
+        free(line);
+      }
+      else
+      {
+        eof = true;
+      }
     }
     else
-    {
-      eof = true;
-    }
-  #else
-    cout << "(oclgrind) " << flush;
-    getline(cin, cmd);
-    eof = cin.eof();
   #endif
+    {
+      if (interactive)
+        cout << "(oclgrind) " << flush;
+      getline(cin, cmd);
+      eof = cin.eof();
+    }
 
     // Quit on EOF
     if (eof)
     {
-      cout << "(quit)" << endl;
+      if (interactive)
+        cout << "(quit)" << endl;
       quit(vector<string>());
       return;
     }
@@ -153,7 +166,8 @@ void InteractiveDebugger::instructionExecuted(
     }
 
   #if HAVE_READLINE
-    add_history(cmd.c_str());
+    if (interactive)
+      add_history(cmd.c_str());
   #endif
 
     // Find command in map and execute
@@ -225,13 +239,7 @@ size_t InteractiveDebugger::getLineNumber(
   llvm::MDNode *md = instruction->getMetadata("dbg");
   if (md)
   {
-#if LLVM_VERSION > 36
-    llvm::DILocation *loc = (llvm::DILocation*)md;
-    return loc->getLine();
-#else
-    llvm::DILocation loc((llvm::MDLocation*)md);
-    return loc.getLineNumber();
-#endif
+    return ((llvm::DILocation*)md)->getLine();
   }
   return 0;
 }
@@ -832,126 +840,15 @@ bool InteractiveDebugger::print(vector<string> args)
   for (unsigned i = 1; i < args.size(); i++)
   {
     cout << args[i] << " = ";
-
-    // Check for subscript operator
-    size_t start = args[i].find("[");
-    if (start != string::npos)
+    try
     {
-      // Find end of subscript
-      size_t end = args[i].find(']');
-      if (end == string::npos)
-      {
-        cout << "missing ']'" << endl;
-        return false;
-      }
-      if (end != args[i].length() - 1)
-      {
-        cout << "invalid variable" << endl;
-        return false;
-      }
-
-      // Parse index value
-      size_t index = 0;
-      string var = args[i].substr(0, start);
-      stringstream ss(args[i].substr(start+1, end-start-1));
-      ss >> index;
-      if (!ss.eof())
-      {
-        cout << "invalid index" << endl;
-        return false;
-      }
-
-      // Get variable value and type
-      const llvm::Value *ptr = workItem->getVariable(var);
-      if (!ptr)
-      {
-        cout << "not found" << endl;
-        return false;
-      }
-
-      const llvm::Type *ptrType = ptr->getType();
-      unsigned addrSpace = ptrType->getPointerAddressSpace();
-
-      // Check for alloca instruction, in which case look at allocated type
-      bool alloca = false;
-      if (ptr->getValueID() == llvm::Value::GlobalVariableVal)
-      {
-        ptrType = ptrType->getPointerElementType();
-      }
-      if (ptr->getValueID() >= llvm::Value::InstructionVal &&
-          ((llvm::Instruction*)ptr)->getOpcode() == llvm::Instruction::Alloca)
-      {
-        ptrType = ((const llvm::AllocaInst*)ptr)->getAllocatedType();
-        if (ptrType->isPointerTy())
-          addrSpace = ptrType->getPointerAddressSpace();
-        alloca = true;
-      }
-
-      // Ensure type is a pointer
-      if (!ptrType->isPointerTy() && !ptrType->isArrayTy())
-      {
-        cout << "not a pointer" << endl;
-        return false;
-      }
-
-      // Get base address
-      size_t base = *(size_t*)workItem->getValueData(ptr);
-      if (alloca)
-      {
-        // Load base address from private memory
-        workItem->getPrivateMemory()->load((unsigned char*)&base,
-                                           base, sizeof(size_t));
-      }
-
-      // Get target memory object
-      Memory *memory = NULL;
-      switch (addrSpace)
-      {
-      case AddrSpacePrivate:
-        memory = workItem->getPrivateMemory();
-        break;
-      case AddrSpaceGlobal:
-      case AddrSpaceConstant:
-        memory = m_context->getGlobalMemory();
-        break;
-      case AddrSpaceLocal:
-        memory = m_kernelInvocation->getCurrentWorkGroup()->getLocalMemory();
-        break;
-      default:
-        cout << "invalid address space" << endl;
-        return false;
-      }
-
-      // Get element type
-      const llvm::Type *elemType = ptrType->getPointerElementType();
-      unsigned elemSize = getTypeSize(elemType);
-
-      // Load data
-      if (!memory->isAddressValid(base + index*elemSize, elemSize))
-      {
-        cout << "invalid memory address" << endl;
-      }
-      else
-      {
-        // Print data
-        void *data = (void*)memory->getPointer(base+index*elemSize);
-        printTypedData(elemType, (unsigned char*)data);
-        cout << endl;
-      }
+      workItem->printExpression(args[i]);
     }
-    else
+    catch (FatalError err)
     {
-      try
-      {
-        if (!workItem->printVariable(args[i]))
-          cout << "not found";
-      }
-      catch (FatalError err)
-      {
-        cout << "not found";
-      }
-      cout << endl;
+      cout << "fatal error: " << err.what();
     }
+    cout << endl;
   }
 
   return false;
